@@ -62,8 +62,8 @@ class rtaudio : public audio {
         {
             AVOIDDENORMALS;
             
-            float* inputs[fDsp->getNumInputs()];
-            float* outputs[fDsp->getNumOutputs()];
+            float** inputs = (float**)alloca(fDsp->getNumInputs() * sizeof(float*));
+            float** outputs = (float**)alloca(fDsp->getNumOutputs() * sizeof(float*));
             
             for (int i = 0; i < fDsp->getNumInputs(); i++) {
                 inputs[i] = &(static_cast<float*>(inbuf))[i * frames];
@@ -92,13 +92,21 @@ class rtaudio : public audio {
                 fDevNumInChans(0), fDevNumOutChans(0) {}
             
         virtual ~rtaudio() 
-        {   
+        {
+#if RTAUDIO_VERSION_MAJOR < 6
             try {
                 fAudioDAC.stopStream();
                 fAudioDAC.closeStream();
             } catch (RtAudioError& e) {
                 std::cout << '\n' << e.getMessage() << '\n' << std::endl;
             }
+#else
+            RtAudioErrorType err = fAudioDAC.stopStream();
+            if (err != RTAUDIO_NO_ERROR) {
+                std::cout << '\n' << fAudioDAC.getErrorText() << '\n' << std::endl;
+            }
+            fAudioDAC.closeStream();
+#endif
         }
         
         virtual bool init(const char* name, dsp* DSP)
@@ -134,17 +142,28 @@ class rtaudio : public audio {
             
             RtAudio::StreamOptions options;
             options.flags |= RTAUDIO_NONINTERLEAVED;
-         
+
+#if RTAUDIO_VERSION_MAJOR < 6
             try {
-                fAudioDAC.openStream(((numOutputs > 0) ? &oParams : NULL), 
-                    ((numInputs > 0) ? &iParams : NULL), FORMAT, 
+                fAudioDAC.openStream(((numOutputs > 0) ? &oParams : NULL),
+                    ((numInputs > 0) ? &iParams : NULL), FORMAT,
                     fSampleRate, &fBufferSize, audioCallback, this, &options);
             } catch (RtAudioError& e) {
                 std::cout << '\n' << e.getMessage() << '\n' << std::endl;
                 return false;
             }
-               
             return true;
+#else
+            RtAudioErrorType err = fAudioDAC.openStream(
+                ((numOutputs > 0) ? &oParams : NULL),
+                ((numInputs > 0) ? &iParams : NULL), FORMAT,
+                fSampleRate, &fBufferSize, audioCallback, this, &options);
+            if (err != RTAUDIO_NO_ERROR) {
+                std::cout << '\n' << fAudioDAC.getErrorText() << '\n' << std::endl;
+                return false;
+            }
+            return true;
+#endif
         }
         
         void setDsp(dsp* DSP)
@@ -163,6 +182,7 @@ class rtaudio : public audio {
         
         virtual bool start() 
         {
+#if RTAUDIO_VERSION_MAJOR < 6
             try {
                 fAudioDAC.startStream();
             } catch (RtAudioError& e) {
@@ -170,15 +190,30 @@ class rtaudio : public audio {
                 return false;
             }
             return true;
+#else
+            RtAudioErrorType err = fAudioDAC.startStream();
+            if (err != RTAUDIO_NO_ERROR) {
+                std::cout << '\n' << fAudioDAC.getErrorText() << '\n' << std::endl;
+                return false;
+            }
+            return true;
+#endif
         }
         
         virtual void stop() 
         {
+#if RTAUDIO_VERSION_MAJOR < 6
             try {
                 fAudioDAC.stopStream();
             } catch (RtAudioError& e) {
                 std::cout << '\n' << e.getMessage() << '\n' << std::endl;
             }
+#else
+            RtAudioErrorType err = fAudioDAC.stopStream();
+            if (err != RTAUDIO_NO_ERROR) {
+                std::cout << '\n' << fAudioDAC.getErrorText() << '\n' << std::endl;
+            }
+#endif
         }
         
         virtual int getBufferSize() 
