@@ -37,13 +37,15 @@
  */
 
 #ifdef _MSC_VER
-typedef void CTree;
+#define CTree_DEFINED
+typedef void CTreeBase;
 #else
-typedef struct {} CTree;
+#define CTree_DEFINED
+typedef struct {} CTreeBase;
 #endif
 
-typedef CTree* Signal;
-typedef CTree* Box;
+typedef CTreeBase* Signal;
+typedef CTreeBase* Box;
 
 enum SType { kSInt, kSReal };
 
@@ -91,12 +93,16 @@ extern "C"
 #endif
     
     /**
-     * Create global compilation context, has to be done first.
+     * Create global compilation context, has to be done first,
+     * and paired with a call to destroyLibContext().
      */
     LIBFAUST_API void createLibContext();
     
     /**
-     * Destroy global compilation context, has to be done last.
+     * Destroy global compilation context, has to be done last,
+     * and paired with a call to createLibContext().
+     * Note that the created DSP factory can be used outside
+     * of the createLibContext/destroyLibContext scope.
      */
     LIBFAUST_API void destroyLibContext();
 
@@ -397,7 +403,7 @@ extern "C"
      *
      * @return the foreign function box.
      */
-    LIBFAUST_API Box CboxFFun(SType rtype, const char** names, SType* atypes, const char* incfile, const char* libfile);
+    LIBFAUST_API Box CboxFFun(enum SType rtype, const char** names, enum SType* atypes, const char* incfile, const char* libfile);
 
     /**
      * Create a foreign constant box.
@@ -408,7 +414,7 @@ extern "C"
      *
      * @return the foreign constant box.
      */
-    LIBFAUST_API Box CboxFConst(SType type, const char* name, const char* incfile);
+    LIBFAUST_API Box CboxFConst(enum SType type, const char* name, const char* incfile);
     
     /**
      * Create a foreign variable box.
@@ -419,7 +425,7 @@ extern "C"
      *
      * @return the foreign variable box.
      */
-    LIBFAUST_API Box CboxFVar(SType type, const char* name, const char* incfile);
+    LIBFAUST_API Box CboxFVar(enum SType type, const char* name, const char* incfile);
     
     /**
      * Generic binary mathematical functions.
